package work.xuye.source.request;

import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import okio.BufferedSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import work.xuye.common.dto.HttpRequestParams;
import work.xuye.common.dto.HttpRes;
import work.xuye.source.properties.SourceProperties;
import work.xuye.source.util.CharsetUtil;

import java.io.IOException;
import java.nio.charset.Charset;
import java.util.concurrent.TimeUnit;

/**
 * @author xuye
 * @since 2023/4/7 16:29
 **/
@Slf4j
@Primary
@Component
public class OkHttpRequestClient implements RequestClient {

    private final Gson gson;
    private final OkHttpClient client;

    @Autowired
    public OkHttpRequestClient(Gson gson, SourceProperties sourceProperties) {
        this.gson = gson;
        SourceProperties.Pool poolConfig = sourceProperties.getPool();
        this.client = new OkHttpClient.Builder()
                .connectTimeout(sourceProperties.getTimeoutSeconds(), TimeUnit.SECONDS)
                .connectionPool(new ConnectionPool(
                        poolConfig.getMaxIdleConnections(),
                        poolConfig.getKeepAliveSeconds(),
                        TimeUnit.SECONDS))
                .followRedirects(true)
                .build();
        log.info("init OkHttpRequestClient success, timeoutSeconds: {}, maxIdleConnections: {}, keepAliveSeconds: {}",
                sourceProperties.getTimeoutSeconds(),
                poolConfig.getMaxIdleConnections(),
                poolConfig.getKeepAliveSeconds());
    }


    @Override
    public HttpRes execute(HttpRequestParams request) {
        Request.Builder requestBuilder = new Request.Builder();
        requestBuilder.url(request.getUrl());
        if (!ObjectUtils.isEmpty(request.getHeaders())) {
            requestBuilder.headers(Headers.Companion.of(request.getHeaders()));
        }
        if (!ObjectUtils.isEmpty(request.getBody())) {
            MediaType mediaType = MediaType.Companion.parse("application/json;charset=utf-8");
            RequestBody requestBody = RequestBody.Companion.create(gson.toJson(request.getBody()), mediaType);
            requestBuilder.method(request.getMethod(), requestBody);
        }
        HttpRes httpRes = new HttpRes();
        Request httpRequest = requestBuilder.build();
        try (Response response = client.newCall(httpRequest).execute()) {
            httpRes.setUrl(request.getUrl());
            httpRes.setStatus(HttpStatus.valueOf(response.code()));
            if (!ObjectUtils.isEmpty(response.body())) {
                BufferedSource source = response.body().source();
                Charset charset = CharsetUtil.getCharsetByName(request.getCharset());
                String responseBody = source.readByteString().string(charset);
                httpRes.setBody(responseBody);
                source.close();
            }
            httpRes.headers(response.headers().toMultimap());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return httpRes;
    }

}
